@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qssign.c,v 1.16 2011/02/10 19:28:56 pbuchbinder Exp $";
+static const char revision[] = "$Id: qssign.c,v 1.19 2011/11/01 22:11:13 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -34,8 +34,6 @@ static const char revision[] = "$Id: qssign.c,v 1.16 2011/02/10 19:28:56 pbuchbi
 #include <regex.h>
 #include <signal.h>
 
-//#include <config.h>
-
 /* openssl */
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
@@ -49,35 +47,51 @@ static const char revision[] = "$Id: qssign.c,v 1.16 2011/02/10 19:28:56 pbuchbi
 #include <apr_file_io.h>
 #include <apr_time.h>
 
-#define MAX_LINE 65536
-#define CR 13
-#define LF 10
+#include "qs_util.h"
+
 #define SEQDIG "12"
 #define MAX_STRING_LEN 32768
 
 #define QS_END "qssign---end-of-data"
 
+static const char *m_fmt = "";
 static long m_nr = 1;
 static int  m_logend = 0;
 static void (*m_end)(const char *) = NULL;
 static int m_end_pos = 0;
 static const char *m_sec = NULL;
 
-static int qs_getLine(char *s, int n) {
-  int i = 0;
-  while (1) {
-    s[i] = (char)getchar();
-    if(s[i] == EOF) return 0;
-    if (s[i] == CR) {
-      s[i] = getchar();
-    }
-    if ((s[i] == 0x4) || (s[i] == LF) || (i == (n - 1))) {
-      s[i] = '\0';
-      return 1;
-    }
-    ++i;
-  }
-}
+typedef struct {
+  const char* fmt;
+  const char* pattern;
+  const char* test;
+} qos_p_t;
+
+#define severity "[A-Z]+"
+
+static const qos_p_t pattern[] = {
+  {
+    "%s | INFO  | "QS_END,
+    "^[0-9]{4}[-][0-9]{2}[-][0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}[ ]+[|][ ]+"severity"[ ]+[|][ ]+[a-zA-Z0-9]+",
+    "2010-04-14 20:18:37,464 | INFO  | org.hibernate.cfg.Configuration"
+  },
+  {
+    "%s INFO  "QS_END,
+    "^[0-9]{4}[-][0-9]{2}[-][0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}[ ]+"severity"[ ]+",
+    "2011-08-30 07:27:22,738 INFO  loginId='test'"
+  },
+  {
+    "%s qssign          end                                      INFO  "QS_END,
+    "^[0-9]{4}[-][0-9]{2}[-][0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}[ ]+[a-zA-Z0-9\\.-]+[ ]+[a-zA-Z0-9\\.-]+[ ]+"severity"[ ]+",
+    "2011-09-01 07:37:17,275 main            org.apache.catalina.startup.Catalina     INFO  Server"
+  },
+  {
+    "%s INFO  "QS_END,
+    "^[0-9]{4}[-][0-9]{2}[-][0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}[ ]+",
+    "2011-08-30 07:27:22,738 "
+  },
+  { NULL, NULL, NULL }
+};
 
 static void qs_write(char *line, int line_size, const char *sec, int sec_len) {
   HMAC_CTX ctx;
@@ -170,6 +184,22 @@ static void qs_end_nj(const char *sec) {
   return;
 }
 
+/* 2010-04-14 20:18:37,464 ... (using m_fmt) */
+static void qs_end_lj(const char *sec) {
+  int sec_len = strlen(sec);
+  char line[MAX_LINE];
+  int dig = atoi(SEQDIG);
+  /* <data> ' ' <sequence number> '#' <hmac>*/
+  int line_size = sizeof(line) - 1 - dig - 1 - (2*HMAC_MAX_MD_CBLOCK) - 1;
+  char time_string[1024];
+  time_t tm = time(NULL);
+  struct tm *ptr = localtime(&tm);
+  strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S,000", ptr);
+  sprintf(line, m_fmt, time_string);
+  qs_write(line, line_size, sec, sec_len);
+  return;
+}
+
 /* Dec  6 04:00:06 localhost kernel: */
 static void qs_end_lx(const char *sec) {
   char hostname[1024];
@@ -206,6 +236,7 @@ void qs_signal_exit(int e) {
  * - 2010 12 03 17:00:30.425 qssign     end        0.0              5-NOTICE:  ..............
  *                                                 46  <- var ->    63      71
  * - Dec  6 04:00:06 localhost kernel:
+ * - some 2010-12-03 17:00:30,425 ...
  */
 static void qs_set_format(char *s) {
   regex_t r_apache_err; 
@@ -225,7 +256,7 @@ static void qs_set_format(char *s) {
     exit(1);
   }
   if(regcomp(&r_nj,
-	     "^[0-9]{4} [0-9]{2} [0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}\\.[0-9]+ [a-zA-Z0-9]+[ ]+.*[A-Z]+[ ]*:",
+	     "^[0-9]{4} [0-9]{2} [0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}\\.[0-9]{3} [a-zA-Z0-9]+[ ]+.*[A-Z]+[ ]*:",
 	     REG_EXTENDED) != 0) {
     fprintf(stderr, "failed to compile regex (nj)\n");
     exit(1);
@@ -252,8 +283,26 @@ static void qs_set_format(char *s) {
     m_end = &qs_end_nj;
   } else if(regexec(&r_lx, s, 0, NULL, 0) == 0) {
     m_end = &qs_end_lx;    
-  } else {
-    /* default */
+  }
+  // search within the generic yyyy-mm-dd hh-mm-ss,mmm patterns
+  if(!m_end) {
+    const qos_p_t *p = pattern;
+    while(p->fmt) {
+      regex_t r_j;
+      if(regcomp(&r_j, p->pattern, REG_EXTENDED) != 0) {
+	fprintf(stderr, "failed to compile regex (%s)\n", p->pattern);
+	exit(1);
+      }
+      if(regexec(&r_j, s, 0, NULL, 0) == 0) {
+	m_fmt = p->fmt;
+	m_end = &qs_end_lj;      
+	break;
+      }
+      p++;
+    }
+  }
+  /* default (apache error log format) */
+  if(m_end == NULL) {
     m_end = &qs_end_apache_err;
   }
   return;
@@ -438,34 +487,84 @@ static char *qs_readpwd(apr_pool_t *pool, const char *prg) {
   return buf;
 }
 
-static void usage(char *cmd) {
+static void usage(char *cmd, int man) {
+  if(man) {
+    //.TH [name of program] [section number] [center footer] [left footer] [center header]
+    printf(".TH %s 1 \"%s\" \"mod_qos utilities %s\" \"%s man page\"\n", qs_CMD(cmd), man_date,
+	   man_version, cmd);
+  }
   printf("\n");
-  printf("Utility to sign/verify log data.\n");
+  if(man) {
+    printf(".SH NAME\n");
+  }
+  qs_man_print(man, "Utility to sign and verify the integrity of log data.\n");
   printf("\n");
-  printf("Usage: %s -s|S <secret> [-e] [-v]\n", cmd);
+  if(man) {
+    printf(".SH SYNOPSIS\n");
+  }
+  qs_man_print(man, "%s%s -s|S <secret> [-e] [-v]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
-  printf("Summary\n");
-  printf("%s is a log data integrity check tool. It reads log data\n", cmd);
-  printf("from stdin (pipe) and writes the signed data to stdout.\n");
+  if(man) {
+    printf(".SH DESCRIPTION\n");
+  } else {
+    printf("Summary\n");
+  }
+  qs_man_print(man, "%s is a log data integrity check tool. It reads log data\n", cmd);
+  qs_man_print(man, "from stdin (pipe) and writes the signed data to stdout.\n");
   printf("\n");
-  printf("Options\n");
-  printf("  -s <secret>\n");
-  printf("     Passphrase used to calculate signature.\n");
-  printf("  -S <program>\n");
-  printf("     Specifies a program which writes the passphrase to stdout.\n");
-  printf("  -e\n");
-  printf("     Writes end marker when stopping data signing.\n");
-  printf("  -v\n");
-  printf("     Verification mode checking the integrity of signed data.\n");
+  if(man) {
+    printf(".SH OPTIONS\n");
+  } else {
+    printf("Options\n");
+  }
+  if(man) printf(".TP\n");
+  qs_man_print(man, "  -s <secret>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Passphrase used to calculate signature.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -S <program>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Specifies a program which writes the passphrase to stdout.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -e\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Writes end marker when stopping data signing.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -v\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Verification mode checking the integrity of signed data.\n");
   printf("\n");
-  printf("Example (sign):\n");
-  printf(" TransferLog \"|./bin/%s -s password -e |./bin/qsrotate -o /var/log/apache/access_log\"\n", cmd);
+  if(man) {
+    printf(".SH EXAMPLE\n");
+    printf("Sign:\n");
+    printf("\n");
+  } else {
+    printf("Example (sign):\n");
+  }
+  qs_man_println(man, " TransferLog \"|./bin/%s -s password -e |./bin/qsrotate -o /var/log/apache/access_log\"\n", cmd);
   printf("\n");
-  printf("Ecample (verify):\n");
-  printf(" cat access_log | %s -s password -v\n", cmd);
+  if(man) {
+    printf("\n");
+    printf("Verify:\n");
+    printf("\n");
+  } else {
+    qs_man_print(man, "Example (verify):\n");
+  }
+  qs_man_println(man, " cat access_log | %s -s password -v\n", cmd);
   printf("\n");
-  printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
-  exit(1);
+  if(man) {
+    printf(".SH SEE ALSO\n");
+    printf("qsexec(1), qsfilter2(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qstail(1)\n");
+    printf(".SH AUTHOR\n");
+    printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
+  } else {
+    printf("See http://opensource.adnovum.ch/mod_qos/ for further details.\n");
+  }
+  if(man) {
+    exit(0);
+  } else {
+    exit(1);
+  }
 }
 
 int main(int argc, const char * const argv[]) {
@@ -496,16 +595,18 @@ int main(int argc, const char * const argv[]) {
     } else if(strcmp(*argv,"-e") == 0) {
       m_logend = 1;
     } else if(strcmp(*argv,"-?") == 0) {
-      usage(cmd);
+      usage(cmd, 0);
     } else if(strcmp(*argv,"-help") == 0) {
-      usage(cmd);
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"--man") == 0) {
+      usage(cmd, 1);
     }
     argc--;
     argv++;
   }
 
   if(m_sec == NULL) {
-    usage(cmd);
+    usage(cmd, 0);
   }
   if(verify) {
     long err = qs_verify(m_sec);
